Trait Stream

pub trait Stream {
    type Item;

    // Required method
    fn poll_next(
        self: Pin<&mut Self>,
        cx: &mut Context<'_>,
    ) -> Poll<Option<Self::Item>>;

    // Provided method
    fn size_hint(&self) -> (usize, Option<usize>) { ... }
}
A stream of values produced asynchronously.

If Future<Output = T> is an asynchronous version of T, then Stream<Item = T> is an asynchronous version of Iterator<Item = T>. A stream represents a sequence of value-producing events that occur asynchronously to the caller.

The trait is modeled after Future, but allows poll_next to be called even after a value has been produced, yielding None once the stream has been fully exhausted.

Required Associated Types

type Item

Values yielded by the stream.

Required Methods

fn poll_next( self: Pin<&mut Self>, cx: &mut Context<'_>, ) -> Poll<Option<Self::Item>>

Attempt to pull out the next value of this stream, registering the current task for wakeup if the value is not yet available, and returning None if the stream is exhausted.

Return value

There are several possible return values, each indicating a distinct stream state:

  • Poll::Pending means that this stream’s next value is not ready yet. Implementations will ensure that the current task will be notified when the next value may be ready.

  • Poll::Ready(Some(val)) means that the stream has successfully produced a value, val, and may produce further values on subsequent poll_next calls.

  • Poll::Ready(None) means that the stream has terminated, and poll_next should not be invoked again.

Panics

Once a stream has finished (returned Ready(None) from poll_next), calling its poll_next method again may panic, block forever, or cause other kinds of problems; the Stream trait places no requirements on the effects of such a call. However, as the poll_next method is not marked unsafe, Rust’s usual rules apply: calls must never cause undefined behavior (memory corruption, incorrect use of unsafe functions, or the like), regardless of the stream’s state.

If this is difficult to guard against then the fuse adapter can be used to ensure that poll_next always returns Ready(None) in subsequent calls.

Provided Methods

fn size_hint(&self) -> (usize, Option<usize>)

Returns the bounds on the remaining length of the stream.

Specifically, size_hint() returns a tuple where the first element is the lower bound, and the second element is the upper bound.

The second half of the tuple that is returned is an Option<usize>. A None here means that either there is no known upper bound, or the upper bound is larger than usize.

Implementation notes

It is not enforced that a stream implementation yields the declared number of elements. A buggy stream may yield less than the lower bound or more than the upper bound of elements.

size_hint() is primarily intended to be used for optimizations such as reserving space for the elements of the stream, but must not be trusted to e.g., omit bounds checks in unsafe code. An incorrect implementation of size_hint() should not lead to memory safety violations.

That said, the implementation should provide a correct estimation, because otherwise it would be a violation of the trait’s protocol.

The default implementation returns (0, None) which is correct for any stream.

Implementations on Foreign Types



impl Stream for ChangedStream



type Item = Changed



fn poll_next( self: Pin<&mut ChangedStream>, cx: &mut Context<'_>, ) -> Poll<Option<<ChangedStream as Stream>::Item>>



impl Stream for InterfacesAddedStream



type Item = InterfacesAdded



fn poll_next( self: Pin<&mut InterfacesAddedStream>, cx: &mut Context<'_>, ) -> Poll<Option<<InterfacesAddedStream as Stream>::Item>>



impl Stream for InterfacesRemovedStream



type Item = InterfacesRemoved



fn poll_next( self: Pin<&mut InterfacesRemovedStream>, cx: &mut Context<'_>, ) -> Poll<Option<<InterfacesRemovedStream as Stream>::Item>>



impl Stream for IntervalStream



type Item = Instant



fn poll_next( self: Pin<&mut IntervalStream>, cx: &mut Context<'_>, ) -> Poll<Option<Instant>>



fn size_hint(&self) -> (usize, Option<usize>)



impl Stream for MessageStream



type Item = Result<Message, Error>



fn poll_next( self: Pin<&mut MessageStream>, cx: &mut Context<'_>, ) -> Poll<Option<<MessageStream as Stream>::Item>>



impl Stream for NameAcquiredStream



type Item = NameAcquired



fn poll_next( self: Pin<&mut NameAcquiredStream>, cx: &mut Context<'_>, ) -> Poll<Option<<NameAcquiredStream as Stream>::Item>>



impl Stream for NameLostStream



type Item = NameLost



fn poll_next( self: Pin<&mut NameLostStream>, cx: &mut Context<'_>, ) -> Poll<Option<<NameLostStream as Stream>::Item>>



impl Stream for NameOwnerChangedStream



type Item = NameOwnerChanged



fn poll_next( self: Pin<&mut NameOwnerChangedStream>, cx: &mut Context<'_>, ) -> Poll<Option<<NameOwnerChangedStream as Stream>::Item>>



impl Stream for OwnerChangedStream<'_>



type Item = Option<UniqueName<'static>>



fn poll_next( self: Pin<&mut OwnerChangedStream<'_>>, cx: &mut Context<'_>, ) -> Poll<Option<<OwnerChangedStream<'_> as Stream>::Item>>



impl Stream for PropertiesChangedStream



type Item = PropertiesChanged



fn poll_next( self: Pin<&mut PropertiesChangedStream>, cx: &mut Context<'_>, ) -> Poll<Option<<PropertiesChangedStream as Stream>::Item>>



impl Stream for SignalStream<'_>



type Item = Message



fn poll_next( self: Pin<&mut SignalStream<'_>>, cx: &mut Context<'_>, ) -> Poll<Option<<SignalStream<'_> as Stream>::Item>>



impl Stream for Timer



type Item = Instant



fn poll_next( self: Pin<&mut Timer>, cx: &mut Context<'_>, ) -> Poll<Option<<Timer as Stream>::Item>>



impl<'a, S, T> Stream for Cloned<S>
where T: 'a + Clone, S: Stream<Item = &'a T>,



type Item = T



fn poll_next( self: Pin<&mut Cloned<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Cloned<S> as Stream>::Item>>



impl<'a, S, T> Stream for Copied<S>
where T: 'a + Copy, S: Stream<Item = &'a T>,



type Item = T



fn poll_next( self: Pin<&mut Copied<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Copied<S> as Stream>::Item>>



impl<'a, T> Stream for PropertyStream<'a, T>
where T: Unpin,



type Item = PropertyChanged<'a, T>



fn poll_next( self: Pin<&mut PropertyStream<'a, T>>, cx: &mut Context<'_>, ) -> Poll<Option<<PropertyStream<'a, T> as Stream>::Item>>



impl<A, B> Stream for Zip<A, B>
where A: Stream, B: Stream,



type Item = (<A as Stream>::Item, <B as Stream>::Item)



fn poll_next( self: Pin<&mut Zip<A, B>>, cx: &mut Context<'_>, ) -> Poll<Option<<Zip<A, B> as Stream>::Item>>



impl<B, S, P> Stream for MapWhile<S, P>
where S: Stream, P: FnMut(<S as Stream>::Item) -> Option<B>,



type Item = B



fn poll_next( self: Pin<&mut MapWhile<S, P>>, cx: &mut Context<'_>, ) -> Poll<Option<<MapWhile<S, P> as Stream>::Item>>



impl<F> Stream for OnceFuture<F>
where F: Future,



type Item = <F as Future>::Output



fn poll_next( self: Pin<&mut OnceFuture<F>>, cx: &mut Context<'_>, ) -> Poll<Option<<OnceFuture<F> as Stream>::Item>>



impl<I> Stream for Iter<I>
where I: Iterator,



type Item = <I as Iterator>::Item



fn poll_next( self: Pin<&mut Iter<I>>, _cx: &mut Context<'_>, ) -> Poll<Option<<Iter<I> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<I> Stream for Iter<I>
where I: Iterator,



type Item = <I as Iterator>::Item



fn poll_next( self: Pin<&mut Iter<I>>, cx: &mut Context<'_>, ) -> Poll<Option<<I as Iterator>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<K, V> Stream for StreamMap<K, V>
where K: Clone + Unpin, V: Stream + Unpin,



type Item = (K, <V as Stream>::Item)



fn poll_next( self: Pin<&mut StreamMap<K, V>>, cx: &mut Context<'_>, ) -> Poll<Option<<StreamMap<K, V> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<P> Stream for Pin<P>
where P: DerefMut + Unpin, <P as Deref>::Target: Stream,



type Item = <<P as Deref>::Target as Stream>::Item



fn poll_next( self: Pin<&mut Pin<P>>, cx: &mut Context<'_>, ) -> Poll<Option<<Pin<P> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<R> Stream for Bytes<R>
where R: AsyncRead + Unpin,



type Item = Result<u8, Error>



fn poll_next( self: Pin<&mut Bytes<R>>, cx: &mut Context<'_>, ) -> Poll<Option<<Bytes<R> as Stream>::Item>>



impl<R> Stream for Lines<R>
where R: AsyncBufRead,



type Item = Result<String, Error>



fn poll_next( self: Pin<&mut Lines<R>>, cx: &mut Context<'_>, ) -> Poll<Option<<Lines<R> as Stream>::Item>>



impl<R> Stream for Split<R>
where R: AsyncBufRead,



type Item = Result<Vec<u8>, Error>



fn poll_next( self: Pin<&mut Split<R>>, cx: &mut Context<'_>, ) -> Poll<Option<<Split<R> as Stream>::Item>>



impl<S> Stream for &mut S
where S: Stream + Unpin + ?Sized,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut &mut S>, cx: &mut Context<'_>, ) -> Poll<Option<<&mut S as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for AssertUnwindSafe<S>
where S: Stream,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut AssertUnwindSafe<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<S as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for ChunksTimeout<S>
where S: Stream,



type Item = Vec<<S as Stream>::Item>



fn poll_next( self: Pin<&mut ChunksTimeout<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<ChunksTimeout<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for Cycle<S>
where S: Stream + Clone,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Cycle<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Cycle<S> as Stream>::Item>>



impl<S> Stream for Drain<'_, S>
where S: Stream + Unpin + ?Sized,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Drain<'_, S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Drain<'_, S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for Enumerate<S>
where S: Stream,



type Item = (usize, <S as Stream>::Item)



fn poll_next( self: Pin<&mut Enumerate<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Enumerate<S> as Stream>::Item>>



impl<S> Stream for Fuse<S>
where S: Stream,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Fuse<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<S as Stream>::Item>>



impl<S> Stream for IntoOrdering<S>
where S: OrderedStream,



type Item = <S as OrderedStream>::Ordering



fn poll_next( self: Pin<&mut IntoOrdering<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<IntoOrdering<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for IntoStream<S>
where S: OrderedStream,



type Item = <S as OrderedStream>::Data



fn poll_next( self: Pin<&mut IntoStream<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<IntoStream<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for IntoTupleStream<S>
where S: OrderedStream,



type Item = (<S as OrderedStream>::Ordering, <S as OrderedStream>::Data)



fn poll_next( self: Pin<&mut IntoTupleStream<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<IntoTupleStream<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for Skip<S>
where S: Stream,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Skip<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Skip<S> as Stream>::Item>>



impl<S> Stream for StepBy<S>
where S: Stream,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut StepBy<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<StepBy<S> as Stream>::Item>>



impl<S> Stream for StreamNotifyClose<S>
where S: Stream,



type Item = Option<<S as Stream>::Item>



fn poll_next( self: Pin<&mut StreamNotifyClose<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<StreamNotifyClose<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for Take<S>
where S: Stream,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Take<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<S as Stream>::Item>>



impl<S> Stream for Timeout<S>
where S: Stream,



type Item = Result<<S as Stream>::Item, Elapsed>



fn poll_next( self: Pin<&mut Timeout<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Timeout<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S> Stream for TimeoutRepeating<S>
where S: Stream,



type Item = Result<<S as Stream>::Item, Elapsed>



fn poll_next( self: Pin<&mut TimeoutRepeating<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<TimeoutRepeating<S> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S, F> Stream for Inspect<S, F>
where S: Stream, F: FnMut(&<S as Stream>::Item),



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Inspect<S, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<Inspect<S, F> as Stream>::Item>>



impl<S, F, Fut> Stream for Then<S, F, Fut>
where S: Stream, F: FnMut(<S as Stream>::Item) -> Fut, Fut: Future,



type Item = <Fut as Future>::Output



fn poll_next( self: Pin<&mut Then<S, F, Fut>>, cx: &mut Context<'_>, ) -> Poll<Option<<Then<S, F, Fut> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S, F, T> Stream for FilterMap<S, F>
where S: Stream, F: FnMut(<S as Stream>::Item) -> Option<T>,



type Item = T



fn poll_next( self: Pin<&mut FilterMap<S, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<FilterMap<S, F> as Stream>::Item>>



impl<S, F, T> Stream for Map<S, F>
where S: Stream, F: FnMut(<S as Stream>::Item) -> T,



type Item = T



fn poll_next( self: Pin<&mut Map<S, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<Map<S, F> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S, P> Stream for Filter<S, P>
where S: Stream, P: FnMut(&<S as Stream>::Item) -> bool,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Filter<S, P>>, cx: &mut Context<'_>, ) -> Poll<Option<<Filter<S, P> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<S, P> Stream for SkipWhile<S, P>
where S: Stream, P: FnMut(&<S as Stream>::Item) -> bool,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut SkipWhile<S, P>>, cx: &mut Context<'_>, ) -> Poll<Option<<SkipWhile<S, P> as Stream>::Item>>



impl<S, P> Stream for TakeWhile<S, P>
where S: Stream, P: FnMut(&<S as Stream>::Item) -> bool,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut TakeWhile<S, P>>, cx: &mut Context<'_>, ) -> Poll<Option<<TakeWhile<S, P> as Stream>::Item>>



impl<S, St, F, B> Stream for Scan<S, St, F>
where S: Stream, F: FnMut(&mut St, <S as Stream>::Item) -> Option<B>,



type Item = B



fn poll_next( self: Pin<&mut Scan<S, St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<B>>



impl<S, U> Stream for Chain<S, U>
where S: Stream, U: Stream<Item = <S as Stream>::Item>,



type Item = <S as Stream>::Item



fn poll_next( self: Pin<&mut Chain<S, U>>, cx: &mut Context<'_>, ) -> Poll<Option<<Chain<S, U> as Stream>::Item>>



impl<S, U> Stream for Flatten<S>
where S: Stream<Item = U>, U: Stream,



type Item = <U as Stream>::Item



fn poll_next( self: Pin<&mut Flatten<S>>, cx: &mut Context<'_>, ) -> Poll<Option<<Flatten<S> as Stream>::Item>>



impl<S, U, F> Stream for FlatMap<S, U, F>
where S: Stream, U: Stream, F: FnMut(<S as Stream>::Item) -> U,



type Item = <U as Stream>::Item



fn poll_next( self: Pin<&mut FlatMap<S, U, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<FlatMap<S, U, F> as Stream>::Item>>



impl<St> Stream for Skip<St>
where St: Stream,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut Skip<St>>, cx: &mut Context<'_>, ) -> Poll<Option<<Skip<St> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St> Stream for Take<St>
where St: Stream,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut Take<St>>, cx: &mut Context<'_>, ) -> Poll<Option<<Take<St> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F> Stream for Filter<St, F>
where St: Stream, F: FnMut(&<St as Stream>::Item) -> bool,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut Filter<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<St as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F> Stream for SkipWhile<St, F>
where St: Stream, F: FnMut(&<St as Stream>::Item) -> bool,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut SkipWhile<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<SkipWhile<St, F> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F> Stream for TakeWhile<St, F>
where St: Stream, F: FnMut(&<St as Stream>::Item) -> bool,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut TakeWhile<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<TakeWhile<St, F> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F, Fut> Stream for Then<St, Fut, F>
where St: Stream, Fut: Future, F: FnMut(<St as Stream>::Item) -> Fut,



type Item = <Fut as Future>::Output



fn poll_next( self: Pin<&mut Then<St, Fut, F>>, cx: &mut Context<'_>, ) -> Poll<Option<<Fut as Future>::Output>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F, T> Stream for FilterMap<St, F>
where St: Stream, F: FnMut(<St as Stream>::Item) -> Option<T>,



type Item = T



fn poll_next( self: Pin<&mut FilterMap<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F, T> Stream for Map<St, F>
where St: Stream, F: FnMut(<St as Stream>::Item) -> T,



type Item = T



fn poll_next( self: Pin<&mut Map<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, F, T> Stream for MapWhile<St, F>
where St: Stream, F: FnMut(<St as Stream>::Item) -> Option<T>,



type Item = T



fn poll_next( self: Pin<&mut MapWhile<St, F>>, cx: &mut Context<'_>, ) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<St, Fut> Stream for StopAfterFuture<St, Fut>
where St: Stream, Fut: Future,



type Item = <St as Stream>::Item



fn poll_next( self: Pin<&mut StopAfterFuture<St, Fut>>, cx: &mut Context<'_>, ) -> Poll<Option<<St as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Empty<T>



type Item = T



fn poll_next( self: Pin<&mut Empty<T>>, _: &mut Context<'_>, ) -> Poll<Option<<Empty<T> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Empty<T>



type Item = T



fn poll_next(self: Pin<&mut Empty<T>>, _: &mut Context<'_>) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Fuse<T>
where T: Stream,



type Item = <T as Stream>::Item



fn poll_next( self: Pin<&mut Fuse<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<T as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Once<T>



type Item = T



fn poll_next(self: Pin<&mut Once<T>>, _: &mut Context<'_>) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Once<T>



type Item = T



fn poll_next(self: Pin<&mut Once<T>>, cx: &mut Context<'_>) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Peekable<T>
where T: Stream,



type Item = <T as Stream>::Item



fn poll_next( self: Pin<&mut Peekable<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<Peekable<T> as Stream>::Item>>



impl<T> Stream for Pending<T>



type Item = T



fn poll_next(self: Pin<&mut Pending<T>>, _: &mut Context<'_>) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Pending<T>



type Item = T



fn poll_next(self: Pin<&mut Pending<T>>, _: &mut Context<'_>) -> Poll<Option<T>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Receiver<T>



type Item = T



fn poll_next( self: Pin<&mut Receiver<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<Receiver<T> as Stream>::Item>>



impl<T> Stream for Receiver<T>
where T: Clone,



type Item = T



fn poll_next( self: Pin<&mut Receiver<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<Receiver<T> as Stream>::Item>>



impl<T> Stream for ReceiverStream<T>



fn size_hint(&self) -> (usize, Option<usize>)

Returns the bounds of the stream based on the underlying receiver.

For open channels, it returns (receiver.len(), None).

For closed channels, it returns (receiver.len(), Some(used_capacity)) where used_capacity is calculated as receiver.max_capacity() - receiver.capacity(). This accounts for any Permit that is still able to send a message.



type Item = T



fn poll_next( self: Pin<&mut ReceiverStream<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<ReceiverStream<T> as Stream>::Item>>



impl<T> Stream for Repeat<T>
where T: Clone,



type Item = T



fn poll_next( self: Pin<&mut Repeat<T>>, _: &mut Context<'_>, ) -> Poll<Option<<Repeat<T> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T> Stream for Unblock<T>
where T: Iterator + Send + 'static, <T as Iterator>::Item: Send + 'static,



type Item = <T as Iterator>::Item



fn poll_next( self: Pin<&mut Unblock<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<T as Iterator>::Item>>



impl<T> Stream for UnboundedReceiverStream<T>



fn size_hint(&self) -> (usize, Option<usize>)

Returns the bounds of the stream based on the underlying receiver.

For open channels, it returns (receiver.len(), None).

For closed channels, it returns (receiver.len(), receiver.len()).



type Item = T



fn poll_next( self: Pin<&mut UnboundedReceiverStream<T>>, cx: &mut Context<'_>, ) -> Poll<Option<<UnboundedReceiverStream<T> as Stream>::Item>>



impl<T, E, F, Fut, Item> Stream for TryUnfold<T, F, Fut>
where F: FnMut(T) -> Fut, Fut: Future<Output = Result<Option<(Item, T)>, E>>,



type Item = Result<Item, E>



fn poll_next( self: Pin<&mut TryUnfold<T, F, Fut>>, cx: &mut Context<'_>, ) -> Poll<Option<<TryUnfold<T, F, Fut> as Stream>::Item>>



impl<T, F> Stream for PollFn<F>
where F: FnMut(&mut Context<'_>) -> Poll<Option<T>>,



type Item = T



fn poll_next(self: Pin<&mut PollFn<F>>, cx: &mut Context<'_>) -> Poll<Option<T>>



impl<T, F> Stream for RepeatWith<F>
where F: FnMut() -> T,



type Item = T



fn poll_next( self: Pin<&mut RepeatWith<F>>, _: &mut Context<'_>, ) -> Poll<Option<<RepeatWith<F> as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T, F, Fut, Item> Stream for Unfold<T, F, Fut>
where F: FnMut(T) -> Fut, Fut: Future<Output = Option<(Item, T)>>,



type Item = Item



fn poll_next( self: Pin<&mut Unfold<T, F, Fut>>, cx: &mut Context<'_>, ) -> Poll<Option<<Unfold<T, F, Fut> as Stream>::Item>>



impl<T, S1, S2> Stream for Or<S1, S2>
where S1: Stream<Item = T>, S2: Stream<Item = T>,



type Item = T



fn poll_next( self: Pin<&mut Or<S1, S2>>, cx: &mut Context<'_>, ) -> Poll<Option<<Or<S1, S2> as Stream>::Item>>



impl<T, S1, S2> Stream for Race<S1, S2>
where S1: Stream<Item = T>, S2: Stream<Item = T>,



type Item = T



fn poll_next( self: Pin<&mut Race<S1, S2>>, cx: &mut Context<'_>, ) -> Poll<Option<<Race<S1, S2> as Stream>::Item>>



impl<T, U> Stream for Chain<T, U>
where T: Stream, U: Stream<Item = <T as Stream>::Item>,



type Item = <T as Stream>::Item



fn poll_next( self: Pin<&mut Chain<T, U>>, cx: &mut Context<'_>, ) -> Poll<Option<<T as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)



impl<T, U> Stream for Merge<T, U>
where T: Stream, U: Stream<Item = <T as Stream>::Item>,



type Item = <T as Stream>::Item



fn poll_next( self: Pin<&mut Merge<T, U>>, cx: &mut Context<'_>, ) -> Poll<Option<<T as Stream>::Item>>



fn size_hint(&self) -> (usize, Option<usize>)

Implementors



impl<A, B> Stream for Either<A, B>
where A: Stream, B: Stream<Item = <A as Stream>::Item>,



type Item = <A as Stream>::Item



impl<A, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::RepeatWith<F>
where F: FnMut() -> A,



type Item = A



impl<B, St, S, Fut, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Scan<St, S, Fut, F>
where St: Stream, F: FnMut(&mut S, <St as Stream>::Item) -> Fut, Fut: Future<Output = Option<B>>,



type Item = B



impl<F> Stream for FlattenStream<F>
where Flatten<F, <F as Future>::Output>: Stream, F: Future,



type Item = <Flatten<F, <F as Future>::Output> as Stream>::Item



impl<F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::future::IntoStream<F>
where Once<F>: Stream,



type Item = <Once<F> as Stream>::Item



impl<Fut> Stream for TryFlattenStream<Fut>
where TryFlatten<Fut, <Fut as TryFuture>::Ok>: Stream, Fut: TryFuture,



type Item = <TryFlatten<Fut, <Fut as TryFuture>::Ok> as Stream>::Item



impl<Fut> Stream for FuturesOrdered<Fut>
where Fut: Future,



type Item = <Fut as Future>::Output



impl<Fut> Stream for FuturesUnordered<Fut>
where Fut: Future,



type Item = <Fut as Future>::Output



impl<Fut> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Once<Fut>
where Fut: Future,



type Item = <Fut as Future>::Output



impl<Fut, Si> Stream for FlattenSink<Fut, Si>
where TryFlatten<Fut, Si>: Stream,



type Item = <TryFlatten<Fut, Si> as Stream>::Item



impl<I> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Iter<I>
where I: Iterator,



type Item = <I as Iterator>::Item



impl<R> Stream for cosmic::iced::daemon::program::graphics::futures::futures::io::Lines<R>
where R: AsyncBufRead,



impl<S> Stream for Box<S>
where S: Stream + Unpin + ?Sized,



type Item = <S as Stream>::Item



impl<S> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Fuse<S>
where S: Stream,



type Item = <S as Stream>::Item



impl<S> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Peekable<S>
where S: Stream,



type Item = <S as Stream>::Item



impl<S> Stream for SplitStream<S>
where S: Stream,



type Item = <S as Stream>::Item



impl<S, F> Stream for SinkMapErr<S, F>
where S: Stream,



type Item = <S as Stream>::Item



impl<S, Item> Stream for Buffer<S, Item>
where S: Sink<Item> + Stream,



type Item = <S as Stream>::Item



impl<S, Item, E> Stream for SinkErrInto<S, Item, E>
where S: Sink<Item> + Stream, <S as Sink<Item>>::Error: Into<E>,



type Item = <S as Stream>::Item



impl<S, Item, U, Fut, F> Stream for With<S, Item, U, Fut, F>
where S: Stream + Sink<Item>, F: FnMut(U) -> Fut, Fut: Future,



type Item = <S as Stream>::Item



impl<S, Item, U, St, F> Stream for WithFlatMap<S, Item, U, St, F>
where S: Stream + Sink<Item>, F: FnMut(U) -> St, St: Stream<Item = Result<Item, <S as Sink<Item>>::Error>>,



type Item = <S as Stream>::Item



impl<St1, St2> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Chain<St1, St2>
where St1: Stream, St2: Stream<Item = <St1 as Stream>::Item>,



type Item = <St1 as Stream>::Item



impl<St1, St2> Stream for Select<St1, St2>
where St1: Stream, St2: Stream<Item = <St1 as Stream>::Item>,



type Item = <St1 as Stream>::Item



impl<St1, St2> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Zip<St1, St2>
where St1: Stream, St2: Stream,



type Item = (<St1 as Stream>::Item, <St2 as Stream>::Item)



impl<St1, St2, Clos, State> Stream for SelectWithStrategy<St1, St2, Clos, State>
where St1: Stream, St2: Stream<Item = <St1 as Stream>::Item>, Clos: FnMut(&mut State) -> PollNext,



type Item = <St1 as Stream>::Item



impl<St> Stream for Abortable<St>
where St: Stream,



type Item = <St as Stream>::Item



impl<St> Stream for BufferUnordered<St>
where St: Stream, <St as Stream>::Item: Future,



type Item = <<St as Stream>::Item as Future>::Output



impl<St> Stream for Buffered<St>
where St: Stream, <St as Stream>::Item: Future,



type Item = <<St as Stream>::Item as Future>::Output



impl<St> Stream for CatchUnwind<St>
where St: Stream + UnwindSafe,



type Item = Result<<St as Stream>::Item, Box<dyn Any + Send>>



impl<St> Stream for Chunks<St>
where St: Stream,



type Item = Vec<<St as Stream>::Item>



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Cycle<St>
where St: Clone + Stream,



type Item = <St as Stream>::Item



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Enumerate<St>
where St: Stream,



type Item = (usize, <St as Stream>::Item)



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Flatten<St>
where Flatten<St, <St as Stream>::Item>: Stream, St: Stream,



type Item = <Flatten<St, <St as Stream>::Item> as Stream>::Item



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::IntoStream<St>
where St: TryStream,



type Item = Result<<St as TryStream>::Ok, <St as TryStream>::Error>



impl<St> Stream for ReadyChunks<St>
where St: Stream,



type Item = Vec<<St as Stream>::Item>



impl<St> Stream for SelectAll<St>
where St: Stream + Unpin,



type Item = <St as Stream>::Item



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Skip<St>
where St: Stream,



type Item = <St as Stream>::Item



impl<St> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Take<St>
where St: Stream,



type Item = <St as Stream>::Item



impl<St> Stream for TryBufferUnordered<St>
where St: TryStream, <St as TryStream>::Ok: TryFuture<Error = <St as TryStream>::Error>,



type Item = Result<<<St as TryStream>::Ok as TryFuture>::Ok, <St as TryStream>::Error>



impl<St> Stream for TryBuffered<St>
where St: TryStream, <St as TryStream>::Ok: TryFuture<Error = <St as TryStream>::Error>,



type Item = Result<<<St as TryStream>::Ok as TryFuture>::Ok, <St as TryStream>::Error>



impl<St> Stream for TryChunks<St>
where St: TryStream,



type Item = Result<Vec<<St as TryStream>::Ok>, TryChunksError<<St as TryStream>::Ok, <St as TryStream>::Error>>



impl<St> Stream for TryFlatten<St>
where St: TryStream, <St as TryStream>::Ok: TryStream, <<St as TryStream>::Ok as TryStream>::Error: From<<St as TryStream>::Error>,



type Item = Result<<<St as TryStream>::Ok as TryStream>::Ok, <<St as TryStream>::Ok as TryStream>::Error>



impl<St> Stream for TryFlattenUnordered<St>
where FlattenUnorderedWithFlowController<NestedTryStreamIntoEitherTryStream<St>, PropagateBaseStreamError<St>>: Stream, St: TryStream, <St as TryStream>::Ok: TryStream + Unpin, <<St as TryStream>::Ok as TryStream>::Error: From<<St as TryStream>::Error>,



type Item = <FlattenUnorderedWithFlowController<NestedTryStreamIntoEitherTryStream<St>, PropagateBaseStreamError<St>> as Stream>::Item



impl<St> Stream for TryReadyChunks<St>
where St: TryStream,



type Item = Result<Vec<<St as TryStream>::Ok>, TryReadyChunksError<<St as TryStream>::Ok, <St as TryStream>::Error>>



impl<St, E> Stream for ErrInto<St, E>
where MapErr<St, IntoFn<E>>: Stream,



type Item = <MapErr<St, IntoFn<E>> as Stream>::Item



impl<St, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Inspect<St, F>
where Map<St, InspectFn<F>>: Stream,



type Item = <Map<St, InspectFn<F>> as Stream>::Item



impl<St, F> Stream for InspectErr<St, F>
where Inspect<IntoStream<St>, InspectErrFn<F>>: Stream,



type Item = <Inspect<IntoStream<St>, InspectErrFn<F>> as Stream>::Item



impl<St, F> Stream for InspectOk<St, F>
where Inspect<IntoStream<St>, InspectOkFn<F>>: Stream,



type Item = <Inspect<IntoStream<St>, InspectOkFn<F>> as Stream>::Item



impl<St, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Map<St, F>
where St: Stream, F: FnMut1<<St as Stream>::Item>,



type Item = <F as FnOnce1<<St as Stream>::Item>>::Output



impl<St, F> Stream for MapErr<St, F>
where Map<IntoStream<St>, MapErrFn<F>>: Stream,



type Item = <Map<IntoStream<St>, MapErrFn<F>> as Stream>::Item



impl<St, F> Stream for MapOk<St, F>
where Map<IntoStream<St>, MapOkFn<F>>: Stream,



type Item = <Map<IntoStream<St>, MapOkFn<F>> as Stream>::Item



impl<St, Fut> Stream for TakeUntil<St, Fut>
where St: Stream, Fut: Future,



type Item = <St as Stream>::Item



impl<St, Fut, F> Stream for AndThen<St, Fut, F>
where St: TryStream, F: FnMut(<St as TryStream>::Ok) -> Fut, Fut: TryFuture<Error = <St as TryStream>::Error>,



type Item = Result<<Fut as TryFuture>::Ok, <St as TryStream>::Error>



impl<St, Fut, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Filter<St, Fut, F>
where St: Stream, F: for<'a> FnMut1<&'a <St as Stream>::Item, Output = Fut>, Fut: Future<Output = bool>,



type Item = <St as Stream>::Item



impl<St, Fut, F> Stream for OrElse<St, Fut, F>
where St: TryStream, F: FnMut(<St as TryStream>::Error) -> Fut, Fut: TryFuture<Ok = <St as TryStream>::Ok>,



type Item = Result<<St as TryStream>::Ok, <Fut as TryFuture>::Error>



impl<St, Fut, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::SkipWhile<St, Fut, F>
where St: Stream, F: FnMut(&<St as Stream>::Item) -> Fut, Fut: Future<Output = bool>,



type Item = <St as Stream>::Item



impl<St, Fut, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::TakeWhile<St, Fut, F>
where St: Stream, F: FnMut(&<St as Stream>::Item) -> Fut, Fut: Future<Output = bool>,



type Item = <St as Stream>::Item



impl<St, Fut, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Then<St, Fut, F>
where St: Stream, F: FnMut(<St as Stream>::Item) -> Fut, Fut: Future,



type Item = <Fut as Future>::Output



impl<St, Fut, F> Stream for TryFilter<St, Fut, F>
where St: TryStream, Fut: Future<Output = bool>, F: FnMut(&<St as TryStream>::Ok) -> Fut,



type Item = Result<<St as TryStream>::Ok, <St as TryStream>::Error>



impl<St, Fut, F> Stream for TrySkipWhile<St, Fut, F>
where St: TryStream, F: FnMut(&<St as TryStream>::Ok) -> Fut, Fut: TryFuture<Ok = bool, Error = <St as TryStream>::Error>,



type Item = Result<<St as TryStream>::Ok, <St as TryStream>::Error>



impl<St, Fut, F> Stream for TryTakeWhile<St, Fut, F>
where St: TryStream, F: FnMut(&<St as TryStream>::Ok) -> Fut, Fut: TryFuture<Ok = bool, Error = <St as TryStream>::Error>,



type Item = Result<<St as TryStream>::Ok, <St as TryStream>::Error>



impl<St, Fut, F, T> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::FilterMap<St, Fut, F>
where St: Stream, F: FnMut1<<St as Stream>::Item, Output = Fut>, Fut: Future<Output = Option<T>>,



type Item = T



impl<St, Fut, F, T> Stream for TryFilterMap<St, Fut, F>
where St: TryStream, Fut: TryFuture<Ok = Option<T>, Error = <St as TryStream>::Error>, F: FnMut(<St as TryStream>::Ok) -> Fut,



type Item = Result<T, <St as TryStream>::Error>



impl<St, U, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::FlatMap<St, U, F>
where Flatten<Map<St, F>, U>: Stream,



type Item = <Flatten<Map<St, F>, U> as Stream>::Item



impl<St, U, F> Stream for FlatMapUnordered<St, U, F>
where FlattenUnorderedWithFlowController<Map<St, F>, ()>: Stream, St: Stream, U: Stream + Unpin, F: FnMut(<St as Stream>::Item) -> U,



type Item = <FlattenUnorderedWithFlowController<Map<St, F>, ()> as Stream>::Item



impl<T> Stream for cosmic::iced::daemon::program::graphics::futures::futures::channel::mpsc::Receiver<T>



type Item = T



impl<T> Stream for UnboundedReceiver<T>



type Item = T



impl<T> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Empty<T>



type Item = T



impl<T> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Pending<T>



type Item = T



impl<T> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Repeat<T>
where T: Clone,



type Item = T



impl<T, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::future::PollImmediate<F>
where F: Future<Output = T>,

A Stream implementation that can be polled repeatedly until the future is done. The stream will never return Poll::Pending so polling it in a tight loop is worse than using a blocking synchronous function.

use core::pin::pin;

use futures::task::Poll;
use futures::{StreamExt, future};
use future::FusedFuture;

let f = async { 1_u32 };
let f = pin!(f);
let mut r = future::poll_immediate(f);
assert_eq!(r.next().await, Some(Poll::Ready(1)));

let f = async {futures::pending!(); 42_u8};
let f = pin!(f);
let mut p = future::poll_immediate(f);
assert_eq!(p.next().await, Some(Poll::Pending));
assert!(!p.is_terminated());
assert_eq!(p.next().await, Some(Poll::Ready(42)));
assert!(p.is_terminated());
assert_eq!(p.next().await, None);


type Item = Poll<T>



impl<T, F> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::PollFn<F>
where F: FnMut(&mut Context<'_>) -> Poll<Option<T>>,



type Item = T



impl<T, F, Fut, Item> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::TryUnfold<T, F, Fut>
where F: FnMut(T) -> Fut, Fut: TryFuture<Ok = Option<(Item, T)>>,



type Item = Result<Item, <Fut as TryFuture>::Error>



impl<T, F, Fut, Item> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::Unfold<T, F, Fut>
where F: FnMut(T) -> Fut, Fut: Future<Output = Option<(Item, T)>>,



type Item = Item



impl<T, S> Stream for cosmic::iced::daemon::program::graphics::futures::futures::prelude::stream::PollImmediate<S>
where S: Stream<Item = T>,



type Item = Poll<T>